@@ -1,6 +1,7 @@
 from __future__ import absolute_import, division, print_function, unicode_literals
 
 from future.moves.itertools import zip_longest
+from future.utils import raise_from
 from builtins import zip, map, range, reversed
 
 import sys, logging, random
@@ -12,12 +13,16 @@ from random import randrange
 from types import GeneratorType
 from typing import TYPE_CHECKING
 
-from .exceptions import IteratorExhausted
+from .exceptions import IteratorExhausted, EmptyIterable
 
 if TYPE_CHECKING:
-	from typing import Any, Callable, Iterable, Iterator, TypeVar, Tuple, Sequence, Optional
+	from typing import (Any, Callable, Iterable, Iterator, TypeVar, Tuple, Sequence,
+		Optional, Collection, List, Union, Type, Dict, TextIO)
 	from logging import Logger
+	from .typing import SizedIterable, Number
 	T = TypeVar("T")
+	U = TypeVar("U")
+	V = TypeVar("V")
 
 logger = logging.getLogger(__name__)
 logger.addHandler(logging.NullHandler())
@@ -44,40 +49,7 @@ def repeatfunc(func, times=None, *args):
 		return starmap(func, repeat(args))
 	return starmap(func, repeat(args, times))
 
-def progressdata(it, refresh=1, end="\r", file=sys.stdout):
-	# type: (Union[Iterable, Sequence], Optional[int], Number, Optional[IO[str]]) -> Iterable
-
-	last = start = time()
-	total = 0
-	try:
-		for elm in it:
-			yield elm
-			total += len(elm)
-			current = time()
-			if current - last > refresh:
-				last = current
-				duration = current-start
-				print("Read {}, running for {} seconds ({:0.2e}/s).".format(total, int(duration), total/duration), end="\r", file=file)
-	except KeyboardInterrupt:
-		print("Unsafely aborted after reading {} in {} seconds ({:0.2e}/s).".format(total, int(last-start), total/(last-start)), end=end, file=file)
-		raise
-	else:
-		duration = last-start
-		if duration > 0:
-			print("Finished {} in {} seconds ({:0.2e}/s).".format(total, int(duration), total/duration), end=end, file=file)
-		else:
-			print("Finished {} in {} seconds.".format(total, int(duration)), end=end, file=file)
-
-def progress(it, length=None, refresh=1, end="\r", file=sys.stdout, extra_info_callback=None):
-	# type: (Union[Iterable, Collection], Optional[int], float, Optional[TextIO], Callable) -> Iterator
-
-	""" Wraps an iterable `it` to periodically print the progress every `refresh` seconds. """
-
-	""" todo: `from operator import length_hint`
-		Use `length_hint(it)` for a guess which might be better than nothing.
-		Can return 0.
-	"""
-
+def _lstr(it, length=None):
 	if length is None:
 		try:
 			length = len(it) # type: ignore # TypeError caught below
@@ -89,10 +61,49 @@ def progress(it, length=None, refresh=1, end="\r", file=sys.stdout, extra_info_c
 	else:
 		lstr = ""
 
-	extra = ""
+	return length, lstr
 
+def progressdata(it, length=None, refresh=1, end="\r", file=sys.stdout):
+	# type: (Union[Iterable, Sequence], Optional[int], float, str, Optional[TextIO]) -> Iterable
+
+	length, lstr = _lstr(it, length)
+	last = start = time()
+	total = 0
+
+	try:
+		for elm in it:
+			yield elm
+			total += len(elm)
+			current = time()
+			if current - last > refresh:
+				last = current
+				duration = current-start
+				print("Read {}{}, running for {} seconds ({:0.2e}/s).".format(total, lstr, int(duration), total/duration), end="\r", file=file)
+	except KeyboardInterrupt:
+		print("Unsafely aborted after reading {}{} in {} seconds ({:0.2e}/s).".format(total, lstr, int(last-start), total/(last-start)), end=end, file=file)
+		raise
+	else:
+		duration = last-start
+		if duration > 0:
+			print("Finished {} in {} seconds ({:0.2e}/s).".format(total, int(duration), total/duration), end=end, file=file)
+		else:
+			print("Finished {} in {} seconds.".format(total, int(duration)), end=end, file=file)
+
+def progress(it, length=None, refresh=1, end="\r", file=sys.stdout, extra_info_callback=None):
+	# type: (Union[Iterable, Collection], Optional[int], float, str, Optional[TextIO], Callable) -> Iterator
+
+	""" Wraps an iterable `it` to periodically print the progress every `refresh` seconds. """
+
+	""" todo: `from operator import length_hint`
+		Use `length_hint(it)` for a guess which might be better than nothing.
+		Can return 0.
+	"""
+
+	length, lstr = _lstr(it, length)
+	extra = ""
 	last = start = time()
 	i = 0
+
 	for i, elm in enumerate(it, 1):
 		yield elm
 		current = time()
@@ -175,8 +186,15 @@ def asc_peaks(iterable):
 
 def extrema(iterable, first=None, last=None, derivatives={1, -1}):
 	it = iter(iterable)
-	old = next(it)
-	old_d = diffsgn(old, next(it))
+	try:
+		old = next(it)
+	except StopIteration:
+		return
+	try:
+		old_d = diffsgn(old, next(it))
+	except StopIteration:
+		yield old # fixme: should this be returned?
+		return
 
 	if old_d == first:
 		yield old
@@ -216,17 +234,35 @@ def empty():
 	return
 	yield # pylint: disable=unreachable
 
-def last(it):
+def lastdefault(it, default=None):
+	# type: (Iterable[T], Optional[T]) -> Optional[T]
 
-	ret = None
+	""" Returns the last element of iterable `it`. It discards all other values.
+		This method is about 1/2 times as fast as `last()` for large iterables.
+	"""
+
+	ret = default
 	for i in it:
 		ret = i
 	return ret
 
+# was: iterlast
+def last(it):
+	# type: (Iterable[T], ) -> T
+
+	""" Returns the last element of iterable `it`. It discards all other values.
+		This method is about 2 times as fast as `lastdefault()` for large iterables.
+	"""
+
+	try:
+		return deque(it, 1).pop()
+	except IndexError:
+		raise_from(EmptyIterable("Empty iterable"), None)
+
 def batch(it, n, filter=None):
 	# type: (Iterable[Any], int, Optional[Callable]) -> Iterator[Any] # return type cannot be more specific because of filter()
 
-	""" Batches iterator `it` into batches of size `n`.
+	""" Batches iterable `it` into batches of size `n`.
 		Optionally post-processes batch with `filter`.
 	"""
 
@@ -243,7 +279,8 @@ def batch(it, n, filter=None):
 			yield chain((first_el,), chunk_it)
 
 def advance(it, n):
-	# type: (Iterable, int) -> None
+	# type: (Iterable[T], int) -> None
+
 	""" Advances the iterable `it` n steps. """
 
 	"""
@@ -263,6 +300,7 @@ def all_equal(it):
 	# type: (Iterable, ) -> bool
 
 	""" Returns `True` if all elements of `it` are equal to each other.
+
 		Much faster than:
 			return len(set(it)) in (0, 1)
 		and also faster than:
@@ -282,27 +320,33 @@ def pairwise(it):
 
 	""" Return two elements of iterable `it` at a time,
 		but only advances the iterable by 1 each time.
-		(a, b, c) -> (a, b), (b, c)
+		pairwise((a, b, c)) -> (a, b), (b, c)
 	"""
 
 	a, b = tee(it, 2)
 	next(b, None)
 	return zip(a, b)
 
-def findfirst(func, it, default_index=None, default_value=None):
-	# type: (Callable, Iterable[T]) -> Optional[T]
+def findfirst(func, it, default=(None, None)):
+	# type: (Callable[[T], bool], Iterable[T], Tuple[Optional[int], Optional[T]]) -> Tuple[Optional[int], Optional[T]]
+
+	""" Find the first element of iterable `it` where `func(elm)` evaluates to True.
+		Return `default` if not such element was found.
+	"""
 
 	for i, x in enumerate(it):
 		if func(x):
 			return i, x
 
-	return default_index, default_value
+	return default
 
 #was: isempty
 def is_empty(it):
-	# type: (Iterable, ) -> bool
+	# type: (Iterator, ) -> bool
 
-	""" Returns True if the iterable `it` is already fully consumed. """
+	""" Returns True if the iterator `it` is already fully consumed.
+		If not, it will be advanced.
+	"""
 
 	try:
 		next(it)
@@ -313,18 +357,26 @@ def is_empty(it):
 def consume(it):
 	# type: (Iterable, ) -> None
 
-	""" Consumes the iterable `it` completely """
+	""" Consumes the iterable `it` completely. """
 
 	deque(it, maxlen=0)
 
 def resizer(it, size, pad=False, pad_elm=None):
 	# type: (Iterable[Sequence[T]], int, bool, Optional[T]) -> Iterable
 
+	""" Cuts the input iterable `it` consisting of variable length slices
+		into slices of length `size`. If pad is True, the last slice
+		will be padded to length `size` with `pad_elm`.
+
+		resizer(("asd", "qw", "e"), 4) -> (['a', 's', 'd', 'q'], ['w', 'e'])
+		resizer(("asd", "qw", "e"), 4, True, "x") -> (['a', 's', 'd', 'q'], ['w', 'e', 'x', 'x'])
+	"""
+
 	if size <= 0:
 		raise ValueError("size must be larger than 0")
 
 	it = iter(it)
-	buf = ""
+	buf = "" # type: Sequence[T]
 	pos = 0
 	try:
 		while True:
@@ -355,7 +407,7 @@ def resizer(it, size, pad=False, pad_elm=None):
 def switch(it):
 	# type: (Iterable[Tuple[T, U]], ) -> Iterator[Tuple[U, T]]
 
-	""" Swaps the elements in a iterable of pairs.
+	""" Swaps the elements in iterable of pairs `it`.
 		((3, 6), (7, 4), (1, 9)) -> (6, 3), (4, 7), (9, 1)
 	"""
 
@@ -364,6 +416,9 @@ def switch(it):
 def switched_enumerate(it):
 	# type: (Iterable[T], ) -> Iterator[Tuple[T, int]]
 
+	""" Same as `enumerate()` except that order of the output pair is switched.
+	"""
+
 	return switch(enumerate(it))
 
 def multi_join(it_a, it_b, join_func=add):
@@ -404,7 +459,7 @@ def list_except(it, catch=Exception):
 	# type: (Iterable[T], Union[Exception, Sequence[Exception]]) -> Tuple[Exception, List[T]]
 
 	""" Same as `list()` except in the case of an exception, the partial list which was collected
-		so far is returned. `catch` specifies which exeptions are caught. It can be an exception
+		so far is returned. `catch` specifies which exceptions are caught. It can be an exception
 		or a tuple of exceptions.
 	"""
 
@@ -453,15 +508,15 @@ def decompress(selectors, data, default=None):
 		raise IteratorExhausted
 
 #was: get_first_non_none
-def first_not_none(it):
-	# type: (Iterable[T], ) -> Optional[T]
+def first_not_none(it, default=None):
+	# type: (Iterable[T], Optional[T]) -> Optional[T]
 
 	""" Returns the first element of `it` which is not None. """
 
-	return next(filternone(it), None)
+	return next(filternone(it), default)
 
 def range_count(start=0, stop=None, step=1):
-	# type: (int, Optional[int]) -> Iterator[int]
+	# type: (int, Optional[int], int) -> Iterator[int]
 
 	""" Similar to `range`, except it optionally allows for an infinite range. """
 
@@ -479,31 +534,41 @@ def product_range_repeat(depth, args):
 
 	return product(tuple(range(*args)), repeat=depth)
 
-def powerset(it):
-	# type: (Iterable[T], ) -> Iterator[Tuple[T, ...]]
+def powerset(sit):
+	# type: (SizedIterable[T], ) -> Iterator[Tuple[T, ...]]
 
-	""" powerset([1,2,3]) --> () (1,) (2,) (3,) (1,2) (1,3) (2,3) (1,2,3) """
+	""" Returns the powerset of a sized iterable.
+		The powerset of an empty set is a set including the empty set.
 
-	s = tuple(it)
-	return chain.from_iterable(combinations(s, r) for r in range(len(s)+1))
+		powerset([1,2,3]) --> (), (1,) (2,) (3,) (1,2) (1,3) (2,3) (1,2,3)
+	"""
+
+	return chain.from_iterable(combinations(sit, r) for r in range(len(sit)+1))
 
 def any_in_common(a, b):
-	# type: (Iterable, Iterable) -> bool
+	# type: (Iterable[T], Iterable[T]) -> bool
 
-	""" Tests if iterables `a` and `b` have any elements in common. """
+	""" Tests if iterables `a` and `b` have any elements in common.
+	"""
 
 	return any(i == j for i, j in product(a, b))
 
 # had cmp=b"\0" default before
-def all_equal_to(iterable, cmp):
+def all_equal_to(it, cmp):
 	# type: (Iterable[T], T) -> bool
 
-	return all(elm == cmp for elm in iterable)
+	""" Test if every item of iterable `it` is equal to `cmp`.
+	"""
+
+	return all(elm == cmp for elm in it)
 
 # was: iter_same
 def iter_equal(*its):
 	# type: (*Iterable, ) -> bool
 
+	""" Test if the contents of all iterables `its` is the same.
+	"""
+
 	return all(all_equal(it_elements) for it_elements in zip_longest(*its))
 
 def iter_different(it_a, it_b):
@@ -515,6 +580,7 @@ def iter_different(it_a, it_b):
 
 def every_n(it, n, pos=0):
 	# type: (Iterable[T], int, int) -> T
+
 	""" Yields every `n`-th element from iterable `it` and starts at `pos`. """
 
 	if pos > 0:
@@ -527,16 +593,18 @@ def every_n(it, n, pos=0):
 def split(it, size):
 	# type: (Iterable[T], int) -> Sequence[Iterable[T]]
 
-	""" memory consumption should be proportional to size when output iterators
-		are used in parallel loop (zip), else might be higher
+	""" memory consumption should be proportional to `size` when output iterators
+		are used in parallel loops (like `zip`), else might be higher
 	"""
 
 	copies = tee(it, size)
 	return tuple(every_n(it, size, pos) for it, pos in zip(copies, range(size)))
 
 def remove_all_dupes(it): # was: remove_dup_list
-	# type: (Iterable, ) -> Iterable
+	# type: (Iterable[T], ) -> Iterable[T]
+
 	""" Removes all duplicates from `it` while preserving order. """
+
 	# Dave Kirby
 	seen = set()
 	return (x for x in it if x not in seen and not seen.add(x))
@@ -555,7 +623,7 @@ def retrier(waittime, attempts=-1, multiplier=1, jitter=0, max_wait=None, jitter
 	elif jitter_dist == "normal":
 		rand = random.normalvariate
 	else:
-		raise ValueError()
+		raise ValueError("Unsupported jitter_dist")
 
 	if attempts == 0:
 		return
@@ -578,9 +646,3 @@ def retrier(waittime, attempts=-1, multiplier=1, jitter=0, max_wait=None, jitter
 
 		waittime *= multiplier
 		jitter *= multiplier
-
-if __name__ == "__main__":
-	import timeit
-	print(min(timeit.repeat(stmt="ae(range(100000))", setup="from __main__ import all_equal as ae")))
-
-	print(min(timeit.repeat(stmt="ae(itertools.repeat(0, 100000))", setup="import itertools; from __main__ import all_equal as ae", number=1000)))
